import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Le programme simule le jeu de vie.
 * Dans le menu Edition, on peut vider la grille, la changer de taille,
 * la remplir aléatoirement avec un taux d'occupation et on peut
 * la remplir à partir d'un fichier.
 * Dans le menu Lecture, on peut lancer la simulation et la stopper
 * on peut également changer la vitesse
 * Le bouton suivant permet de faire du pas a pas.
 * Le bouton taux d'occupation affiche une fenêtre qui dessine une courbe
 * du taux d'occupation en fonction du temps
 */
public class UI extends JFrame {
    private Grille grille;
    private final UI fenetre = this;
    private JPanel p;
    private final ThreadLecture t;
    private final ArrayList<Integer> listeTaux;
    private final Courbe c;

    public UI() {
        super("Jeu de la vie!");
        grille = new Grille(10, 50);
        p = afficheGrille();
        listeTaux = new ArrayList<>();
        c = new Courbe(this);
        add(p);
        t = new ThreadLecture(fenetre);
        menu();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setSize(1000, 1000);
        setLocationRelativeTo(null);
        setVisible(true);
    }

    private void menu() {
        //La barre de menu
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        //Menu Edition
        JMenu menuEdition = new JMenu("Edition");
        JMenuItem menuItemGrilleVide = new JMenuItem("Grille Vide");
        menuItemGrilleVide.addActionListener(e -> {
            grille = new Grille(grille.getTaille(), 0);
            updateFenetre();
        });
        menuEdition.add(menuItemGrilleVide);

        JMenu menuTailleGrille = new JMenu("Taille de la grille");
        JTextField menuLargeur = new JTextField(10);
        menuLargeur.addActionListener(e -> {
            //Effet de changer la taille de la grille
            String input = menuLargeur.getText();
            if (input.matches("[0-9]+")) {
                grille = new Grille(Integer.parseInt(input), 0);
                updateFenetre();
            }
        });
        menuTailleGrille.add(new JLabel("Taille:"));
        menuTailleGrille.add(menuLargeur);
        menuEdition.add(menuTailleGrille);

        JMenu menuGrilleAlea = new JMenu("Grille aléatoire");
        JTextField menuTauxOccupation = new JTextField();
        menuTauxOccupation.addActionListener(e -> {
            String input = menuTauxOccupation.getText();
            if (input.matches("[0-9]+")) {
                grille = new Grille(grille.getTaille(), Integer.parseInt(input));
                updateFenetre();
            }
        });
        menuGrilleAlea.add(new JLabel("Taux d'occupation:"));
        menuGrilleAlea.add(menuTauxOccupation);
        menuEdition.add(menuGrilleAlea);

        JMenuItem menuGrillePreRemplie = new JMenuItem("Grille pré-remplie");
        JFileChooser fc = new JFileChooser();
        menuGrillePreRemplie.addActionListener(e -> {
            int valRetour = fc.showOpenDialog(this);
            if (valRetour == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                //Effet de modifier la grille en fonction du fichier sélectionné
                try {
                    grille = new Grille(grille.getTaille(), file);
                    updateFenetre();
                } catch (IOException er) {
                    er.printStackTrace();
                }
            }
        });
        menuEdition.add(menuGrillePreRemplie);
        menuBar.add(menuEdition);


        //Menu Lecture
        JMenu menuLecture = new JMenu("Lecture");
        JMenuItem menuStart = new JMenuItem("Start");
        menuStart.addActionListener(e -> {
            //Effet de passer à l'étape suivante de facon périodique
            t.setLecture(true);
        });
        menuLecture.add(menuStart);
        JMenuItem menuStop = new JMenuItem("Stop");
        menuStop.addActionListener(e -> {
            //Effet d'arreter la lecture automatique
            t.setLecture(false);
        });
        menuLecture.add(menuStop);
        JMenu menuVitesse = new JMenu("Vitesse");
        menuVitesse.add(new JLabel("Nombre d'itération par seconde"));
        ButtonGroup groupVitesse = new ButtonGroup();
        JRadioButtonMenuItem vitesse0 = new JRadioButtonMenuItem("x0.5");
        JRadioButtonMenuItem vitesse1 = new JRadioButtonMenuItem("x1");
        JRadioButtonMenuItem vitesse2 = new JRadioButtonMenuItem("x2");
        JRadioButtonMenuItem vitesse5 = new JRadioButtonMenuItem("x5");
        JRadioButtonMenuItem vitesse10 = new JRadioButtonMenuItem("x10");
        JRadioButtonMenuItem vitesse30 = new JRadioButtonMenuItem("x30");

        vitesse1.setSelected(true);

        groupVitesse.add(vitesse0);
        menuVitesse.add(vitesse0);
        vitesse0.addActionListener(e -> {
            //Définit la vitesse sur 0.5
            t.setVitesse(0.5);
        });

        groupVitesse.add(vitesse1);
        menuVitesse.add(vitesse1);
        vitesse1.addActionListener(e -> {
            //Définit la vitesse sur 1
            t.setVitesse(1);
        });

        groupVitesse.add(vitesse2);
        menuVitesse.add(vitesse2);
        vitesse2.addActionListener(e -> {
            //Définit la vitesse sur 2
            t.setVitesse(2);
        });

        groupVitesse.add(vitesse5);
        menuVitesse.add(vitesse5);
        vitesse5.addActionListener(e -> {
            //Définit la vitesse sur 5
            t.setVitesse(5);
        });

        groupVitesse.add(vitesse10);
        menuVitesse.add(vitesse10);
        vitesse10.addActionListener(e -> {
            //Définit la vitesse sur 10
            t.setVitesse(10);
        });

        groupVitesse.add(vitesse30);
        menuVitesse.add(vitesse30);
        vitesse30.addActionListener(e -> {
            //Définit la vitesse sur 30
            t.setVitesse(30);
        });
        menuLecture.add(menuVitesse);
        menuBar.add(menuLecture);

        JMenuItem menuSuivant = new JMenuItem("Suivant");
        menuSuivant.addActionListener(e -> {
            next();
            updateGrille();
        });
        menuBar.add(menuSuivant);

        JMenuItem menuTaux = new JMenuItem("Taux d'occupation");
        menuTaux.addActionListener(e -> c.setVisible(!c.isVisible()));


        menuBar.add(menuTaux);

    }

    private JPanel afficheGrille() {
        JPanel grille = new JPanel(new GridLayout(this.grille.getTaille(), this.grille.getTaille()));

        Border blackline = BorderFactory.createLineBorder(Color.black, 1);

        for (int lig = 0; lig < this.grille.getTaille(); lig++) {
            for (int col = 0; col < this.grille.getTaille(); col++) {
                Case c = new Case(fenetre, lig, col);
                c.setBorder(blackline);
                grille.add(c);
            }
        }

        grille.setBorder(blackline);

        return grille;
    }

    private void updateGrille() {
        for (Component c : p.getComponents())
            if (c instanceof Case) ((Case) c).color();
    }

    private void updateFenetre() {
        fenetre.remove(p);
        p = afficheGrille();
        fenetre.add(p);
        fenetre.revalidate();
        fenetre.repaint();
        updateGrille();
    }

    private void next() {
        grille.next();
        listeTaux.add(grille.tauxOccupation());
        while (listeTaux.size() > 30)
            listeTaux.remove(0);
        c.repaint();
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(UI::new);
    }

    private static class Case extends JButton {
        private final int lig;
        private final int col;
        private final UI f;

        Case(UI f, int lig, int col) {
            this.lig = lig;
            this.col = col;
            this.f = f;
            this.setContentAreaFilled(false);
            this.addActionListener(e -> trigger());
            color();
            setOpaque(true);
        }

        void trigger() {
            f.grille.swap(lig, col);
            color();
        }

        void color() {
            if (f.grille.getCase(lig, col) == 1)
                setBackground(Color.black);
            else
                setBackground(Color.white);
        }
    }

    private static class ThreadLecture {
        private boolean lecture;
        private double vitesse;
        private final UI f;
        private java.util.Timer t;

        public ThreadLecture(UI f) {
            lecture = false;
            vitesse = 1;
            this.f = f;
            setVitesse(1);
        }

        public void setLecture(boolean b) {
            lecture = b;
        }

        public void setVitesse(double v) {
            vitesse = v;
            if (t != null) t.cancel();
            t = new Timer();
            t.schedule(new TimerTask() {
                @Override
                public void run() {
                    if (lecture) {
                        f.next();
                        f.updateFenetre();
                    }
                }
            }, 0, (long) (1000 / vitesse));
        }
    }

    private static class Courbe extends JFrame {

        Courbe(UI f) {
            super("Courbe du taux d'occupation");
            setSize(400, 200);
            setLocationRelativeTo(null);
            add(new JPanel() {
                protected void paintComponent(Graphics graphicsSimple) {
                    super.paintComponent(graphicsSimple);
                    Graphics2D graphics = (Graphics2D) graphicsSimple;
                    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                    //background
                    graphics.setColor(Color.WHITE);
                    graphics.fillRect(0, 0, getWidth(), getHeight());
                    //axe axis
                    graphics.setColor(Color.GRAY);
                    graphics.drawLine(10, getHeight() - 10, getWidth()-10, getHeight() - 10);
                    graphics.drawLine(10, 1, 10, getHeight() - 10);
                    // --- Draw values ---
                    graphics.setColor(Color.BLACK);
                    graphics.drawString("100%", 0, 10);
                    graphics.drawString("0%", 0, getHeight());

                    //draw courbe
                    if (f.listeTaux.size() > 0) {
                        double hauteur = getHeight() - 10;
                        double largeur = getWidth() - 10;
                        double intervalle = largeur / f.listeTaux.size();
                        int xp = 0;
                        int yp = (int) (hauteur - f.listeTaux.get(0) * hauteur / 100);
                        for (int i = 1; i < f.listeTaux.size(); i++) {
                            int xs = (int) (i * intervalle);
                            int ys = (int) (hauteur - f.listeTaux.get(i) * hauteur / 100);
                            graphics.drawLine(xp + 10, yp, xs + 10, ys);
                            xp = xs;
                            yp = ys;
                        }
                    }
                }
            });
        }
    }
}